#!/bin/sh

set -o pipefail

ARCH="$(ls /var/www/hiawatha/upload/update[0-9][0-9][0-9].tar.gz 2>/dev/null | head -1)"
CFGBACKUP="$(ls -lt /var/www/hiawatha/upload/backup*.tar.gz 2>/dev/null | awk '{print $9}' | head -1)"
RCCMDCERT="$(ls -lt /var/www/hiawatha/upload/rccmd.pem 2>/dev/null | awk '{print $9}' | head -1)"
SERVERCERT="$(ls -lt /var/www/hiawatha/upload/server.pem 2>/dev/null | awk '{print $9}' | head -1)"
LOGOGIF="$(ls -lt /var/www/hiawatha/upload/logo.gif 2>/dev/null | awk '{print $9}' | head -1)"
LOGOSGIF="$(ls -lt /var/www/hiawatha/upload/logos.gif 2>/dev/null | awk '{print $9}' | head -1)"

LOGODST="/var/www/hiawatha/assets/img"

pout() {
  echo $*
}

perr () {
  echo "error: $*"
  echo ":: 1 Update failed."
  exit 1
}

perr_msg () {
  echo ":: 1 $*"
  exit 1
}

get_system () {
   local oob="$(dmesg | grep 'NAND device:' | egrep -o 'OOB size: [0-9]+' | egrep -o '[0-9]+')"

   if [ "${oob}" -eq 64 ]; then
      echo 'bch8'
   elif [ "${oob}" -eq 128 ]; then
      echo 'bch16'
   else
      echo 'none'
   fi
}

run_update() {
   rm -f /tmp/install.sh
   gunzip -c "${ARCH}" | tar -xOf - ./install.sh > /tmp/install.sh
   if [ "$?" -ne 0 ]; then
      perr "Can not extract install.sh from update archive"
   fi

   local sys="$(get_system)"
   local upd='bch16'
   grep -q bch8 /tmp/install.sh
   if [ "$?" -eq 0 ]; then
      upd='bch8'
   fi
   if [ "${sys}" = 'bch8' -a "${upd}" = 'bch16' ]; then
      perr_msg "Cannot downgrade to this version. Please use at least version 1.66."
   fi

   chmod a+x /tmp/install.sh
   exec /tmp/install.sh
}

run_config_restore() {
   if [ ! -x /usr/bin/restoreConfigBackup.sh ]; then
      perr "restoreConfigBackup.sh script not found."
   fi
   exec /usr/bin/restoreConfigBackup.sh
}

run_update_server_cert() {
   cp -f /var/www/hiawatha/upload/server.pem /etc/hiawatha/
   rm /var/www/hiawatha/upload/server.pem
   chmod 644 /etc/hiawatha/server.pem
   chown root:root /etc/hiawatha/server.pem
}

run_update_rccmd_cert() {
   cp -f /var/www/hiawatha/upload/rccmd.pem /etc/gxserve/
   rm /var/www/hiawatha/upload/rccmd.pem
   chmod 644 /etc/gxserve/rccmd.pem
   chown root:root /etc/gxserve/rccmd.pem
}

run_update_logo() {
   if [ ! -f ${1} ]; then
      perr "Cannot find file ${1}."
   fi
   local FNAME=$(basename "${1}")
   local FPATH="${LOGODST}/${FNAME}"
   mv -f ${1} "${FPATH}"
   chmod 644 "${FPATH}"
   pout "Successfully updated ${FNAME}!"
}

ensure_free_space() {
   if [ -z $1 ]; then
      echo "param1 must be desired free space in kBytes."
      return 2
   fi
   if [ -z $2 ]; then
      echo "param2 must be a path in which to ensure free space."
      return 2
   fi

   # delete possible tmp files:
   rm -f /tmp/messages-*

   CDIR=$2
   MIN_FREE=$1
   DFREE=$(df $CDIR | tail -1 | awk '{print $4}')
   if [ "$DFREE" -lt "$MIN_FREE" ]; then
      echo "executing logrotate"
      /etc/cron.hourly/logrotate
   else
      # success! enough free disk space ensured.
      return 0
   fi

   # delete possible networktrace files in existence

   DFREE=$(df $CDIR | tail -1 | awk '{print $4}')
   if [ "$DFREE" -lt "$MIN_FREE" ]; then
	echo "deleting networktrace files"
	rm -f /var/log/networktrace.bin*
   else
  	# success! enough free disk space available.
	return 0
   fi

   # start deleting old, compressed log rotations
   DFREE=$(df $CDIR | tail -1 | awk '{print $4}')
   while [ "$DFREE" -lt "$MIN_FREE" ]; do
      OLDESTFILE=$(ls -lt $CDIR/*.gz | tail -1 | awk '{print $9}')
      if [ "" = "$OLDESTFILE" ]; then
         break;
      fi
      echo "deleting $OLDESTFILE"
      rm $OLDESTFILE
      DFREE=$(df $CDIR | tail -1 | awk '{print $4}')
   done
   if [ "$DFREE" -lt "$MIN_FREE" ]; then
      return 1
   fi
   return 0
}

run_cleanup() {
   FAILED=0
   # ensure enough free disk space in flash file system
   MIN_FREE=50000

   #truncate update.log
   : > /var/log/update.log

   # empty upload directory
   rm -rf /var/www/hiawatha/upload/*

   echo "calling ensure_free_space for flash partition..."
   ensure_free_space "$MIN_FREE" "/var/log"
   RES=$?
   if [ "0" -ne "$RES" ]; then
      echo "failed to clear enough disk space on flash partition RES=$RES"
      FAILED=1
   else
      echo "free space ensured on flash partition..."
   fi
   # ensure free disk space in tempfs
   echo "calling ensure_free_space for tmp partition..."
   ensure_free_space "$MIN_FREE" "/tmp"
   RES=$?
   if [ "0" -ne "$RES" ]; then
      echo "failed to clear enough disk space on tempfs partition RES=$RES"
      FAILED=1
   else
      echo "free space ensured on tempfs partition..."
   fi
   return $FAILED
}

if [ "cleanup" = "$ACTION" ]; then
   run_cleanup > /var/log/cleanup.log 2>&1
   exit $?
elif [ "createConfigBackup" = "$ACTION" ]; then
   exec /usr/bin/createConfigBackup.sh
   exit $?
elif [ "reboot" = "$ACTION" ]; then
   logger reboot by user
   reboot
   exit $?
elif [ "createNetworktraceArchive" == "$ACTION" ]; then
   exec /usr/bin/networkTraceFileArchive.sh
   exit $?
else
   if [ -f "${ARCH}" ]; then
      run_update > /var/log/update.log 2>&1
   elif [ -f "${CFGBACKUP}" ]; then
      run_config_restore
   elif [ -f "${SERVERCERT}" ]; then
      run_update_server_cert > /var/log/server-cert.log 2>&1
   elif [ -f "${RCCMDCERT}" ]; then
      run_update_rccmd_cert > /var/log/rccmd-cert.log 2>&1
   elif [ -f "${LOGOGIF}" ]; then
      run_update_logo "${LOGOGIF}" > /var/log/logoupdate.log 2>&1
   elif [ -f "${LOGOSGIF}" ]; then
      run_update_logo "${LOGOSGIF}" > /var/log/logoupdate.log 2>&1
   else
      perr "Can find no file to work on. Expected either update*.tar.gz or backup*.tar.gz"
   fi
fi
